#include <rxduino.h>
#include <Wire.h>

#define LCD_RST     3
#define LCD_SDA     4
#define LCD_SCL     5

// コマンドを送信します。HD44780でいうRS=0に相当
void i2c_cmd(unsigned char db)
{
     Wire.beginTransmission(0b0111110);
     Wire.write(0b00000000);
     Wire.write(db);
     Wire.endTransmission();
}

// データを送信します。HD44780でいうRS=1に相当
void i2c_data(unsigned char db)
{
     Wire.beginTransmission(0b0111110);
     Wire.write(0b01000000);
     Wire.write(db);
     Wire.endTransmission();
}

// （主に）文字列を連続送信します。
void i2c_puts(const char *s)
{
     while(*s){
          i2c_data(*s++);
     }
}


static unsigned char icon_data[]={
     // アイコンアドレス, 該当ビット
     0x00, 0b10000,
     0x02, 0b10000,
     0x04, 0b10000,
     0x06, 0b10000,
    
     0x07, 0b10000,
     0x07, 0b01000,
     0x09, 0b10000,
     0x0B, 0b10000,
    
     0x0D, 0b01000,
     0x0D, 0b00100,
     0x0D, 0b00010,
     0x0D, 0b10000,
    
     0x0F, 0b10000, // アンテナマーク
};


/*
 *     アイコンの表示ルーチン
 *
 *     それぞれのアイコンに該当するビットを立てる（クリア）します。
 *     現在のアイコンの表示状態は読み取れないため、マイコン側でデータを
 *     編集してからまとめて送信する必要があります。icon_buff[]
 */

// 引き数：各ビットがそれぞれのアイコンに相当します。bit0〜bit12（全１３ビット）
// bit0が右端のアイコン, bit13が左のアイコン（アンテナ）に該当します。
// puticon(0x1FFF)で全アイコン点灯

void puticon(unsigned short flg)
{
     static unsigned char icon_buff[16];     // アイコンの編集用
     unsigned char i;
    
     for(i=0;i<sizeof(icon_data)/2;i++){
          if(flg & (0x1000>>i)){     // 該当ビットが立っていたら
               icon_buff[icon_data[i*2]] |= icon_data[i*2+1];     // バッファを立てます。
          } else {
               icon_buff[icon_data[i*2]] &= ~icon_data[i*2+1];     // バッファをクリアします。
          }
     }
     // 一括でLCDに書き込みます。
     for(i=0;i<16;i++){
          i2c_cmd(0b00111001);     // コマンド
          i2c_cmd(0b01000000+i);     // アイコン領域のアドレスを設定
          i2c_data(icon_buff[i]);     // アイコンデータ
     }
}


void setup(void)
{
     // コントラストの設定
     unsigned char contrast = 0b1011111;     // 数値を上げると濃くなります。
                              // コントラストは電源電圧，温度によりかなり変化します。実際の液晶をみて調整してください。

     pinMode(LCD_RST, OUTPUT);
     Wire.assignSdaScl( LCD_SDA, LCD_SCL ) ;

     delay(500);
    
     digitalWrite(LCD_RST, LOW);           // RSTをLにします。リセット
     delay(1);
     digitalWrite(LCD_RST, HIGH);           // RSTをHにします。リセット解除
     delay(10);
    
     // ここからI2C LCDの初期化を行います。
     delay(40);
     i2c_cmd(0b00111000); // function set
     i2c_cmd(0b00111001); // function set
     i2c_cmd(0b00010100); // interval osc
     i2c_cmd(0b01110000 | (contrast & 0xF)); // contrast Low
    
     i2c_cmd(0b01011100 | ((contrast >> 4) & 0x3)); // contast High/icon/power
     i2c_cmd(0b01101100); // follower control
     delay(300);

     i2c_cmd(0b00111000); // function set
     i2c_cmd(0b00001100); // Display On
    
     i2c_cmd(0b00000001); // Clear Display
     delay(2);                // Clear Displayは追加ウェイトが必要
    
     // １行目の表示
     i2c_puts("Hello World!!");
    
     // ２行目にカーソルを移動
     i2c_cmd(0b11000000);     // ADDR=0x40
    
     // カナの表示（このソースプログラムはSJISで記述されていないとうまく表示されません）
     i2c_puts("Gadget Renesas");
    
}

void loop() {
     char i;
     // アイコンを点滅させる
     while(1){
          for(i=0;i<=12;i++){
               // １箇所だけビットを反転させてアイコンを非表示にします。
               // ビットを立てると表示ＯＮになります。
               puticon(0x1FFF ^ (1<<i));     // １ビットだけ反転
               delay(500);
          }
     }
     
}

